/*******************************************************************
*
*  DESCRIPTION: Atomic Model PackageBuffer
*
*  AUTHOR: Abdullah Alfaify
*
*  EMAIL: mailto://aalfa064@uottawa.ca
*
*  DATE: 14/12/2011
*
*******************************************************************/

/** include files **/
#include "packageBuffer.h"      // class PackageBuffer
#include "message.h"    // class ExternalMessage, InternalMessage
#include "mainsimu.h"   // MainSimulator::Instance().getParameter( ... )

/** public functions **/

/*******************************************************************
* Function Name: PackageBuffer
* Description: 
********************************************************************/
PackageBuffer::PackageBuffer( const string &name )
: Atomic( name )
, in( addInputPort( "in" ) )
, done( addInputPort( "done" ) )
, out( addOutputPort( "out" ) )
, mg( addOutputPort( "mg" ) )
, inform( addOutputPort( "inform" ) )
, preparationTime( 0, 0, 1, 0 )
{	string time( MainSimulator::Instance().getParameter( description(), "preparation" ) ) ;

	if( time != "" )
		preparationTime = time ;
}


/*******************************************************************
* Function Name: initFunction
* Description: in this function we will set the initial values of the Buffer
********************************************************************/
Model &PackageBuffer::initFunction()
{
	// initially we clean up the Buffer.
	elements.erase( elements.begin(), elements.end() ) ;

	return *this ;
}

/*******************************************************************
* Function Name: externalFunction
* Description: 
********************************************************************/
Model &PackageBuffer::externalFunction( const ExternalMessage &msg )
{
	// incoming message to the (in) port
		if( msg.port() == in )
		{
			// add the new element to the back of the buffer.
			elements.push_back( msg.value() ) ;

			if (state != sending){
				state = collecting;
				holdIn( active, Time::Zero ); // call output function to send the size of the buffer to the  Package operation.
			}else {

				state = collecting;
			}


		}

		if( msg.port() == done )
		{
			state = sending;

			if( !elements.empty() )
				holdIn( active,  preparationTime ); // call output function.

		}

		if( msg.port() == in && msg.port() == done )
				{
					elements.push_back( msg.value() ) ;
					state = sending;

					if( !elements.empty() )
						holdIn( active,  preparationTime ); // call output function
					else {
								state = collecting;
								holdIn( active, Time::Zero ); // call output function to send the size of the buffer to the  Package operation.
							}

				}


	return *this;
}

/*******************************************************************
* Function Name: internalFunction
* Description: 
********************************************************************/
Model &PackageBuffer::internalFunction( const InternalMessage & )
{
	passivate();
	return *this ;
}

/*******************************************************************
* Function Name: outputFunction
* Description: 
********************************************************************/
Model &PackageBuffer::outputFunction( const InternalMessage &msg )
{
	switch (state)
			{
				case  collecting :
					// send the buffer size
						sendOutput( msg.time(), inform, elements.size() ) ;
						sendOutput( msg.time(), mg, elements.size() ) ;
						break;

				case sending :
					// send the element
					sendOutput( msg.time(), out, elements.front() ) ;

					elements.pop_front() ;// remove the element after send.

					sendOutput( msg.time(), mg, elements.size() ) ;

					break;


			}

	return *this ;
}
